/**********************************************************************/

#ifndef CLKSYS_DRIVER_H
#define CLKSYS_DRIVER_H

#include "avr_compiler.h"


/* Definitions of macros. */

/*! \brief This macro enables the selected oscillator.
 *
 *  \note Note that the oscillator cannot be used as a main system clock
 *        source without being enabled and stable first. Check the ready flag
 *        before using the clock. The function CLKSYS_IsReady( _oscSel )
 *        can be used to check this.
 *
 *  \param  _oscSel Bitmask of selected clock. Can be one of the following
 *                  OSC_RC2MEN_bm, OSC_RC32MEN_bm, OSC_RC32KEN_bm, OSC_XOSCEN_bm,
 *                  OSC_PLLEN_bm.
 */
#define CLKSYS_Enable( _oscSel ) ( OSC.CTRL |= (_oscSel) )

/*! \brief This macro check if selected oscillator is ready.
 *
 *  This macro will return non-zero if is is running, regardless if it is
 *  used as a main clock source or not.
 *
 *  \param _oscSel Bitmask of selected clock. Can be one of the following
 *                 OSC_RC2MEN_bm, OSC_RC32MEN_bm, OSC_RC32KEN_bm, OSC_XOSCEN_bm,
 *                 OSC_PLLEN_bm.
 *
 *  \return  Non-zero if oscillator is ready and running.
 */
#define CLKSYS_IsReady( _oscSel ) ( OSC.STATUS & (_oscSel) )

/*! \brief This macro disables routing of clock signals to the Real-Time
 *         Counter (RTC).
 *
 *  Disabling the RTC saves power if the RTC is not in use.
 */
#define CLKSYS_RTC_ClockSource_Disable() ( CLK.RTCCTRL &= ~CLK_RTCEN_bm )

/*! \brief This macro disables the automatic calibration of the selected
 *         internal oscillator.
 *
 *  \param _clk  Clock source calibration to disable, either DFLLRC2M or DFLLRC32M.
 */
#define CLKSYS_AutoCalibration_Disable( _clk ) ( (_clk).CTRL &= ~DFLL_ENABLE_bm )


/* Prototyping of function. Detailed information is found in source file. */
void CCPWrite( volatile uint8_t * address, uint8_t value );
void CLKSYS_XOSC_Config( OSC_FRQRANGE_t freqRange,
                         bool lowPower32kHz,
                         OSC_XOSCSEL_t xoscModeSelection );
void CLKSYS_PLL_Config( OSC_PLLSRC_t clockSource, uint8_t factor );
uint8_t CLKSYS_Disable( uint8_t oscSel );
void CLKSYS_Prescalers_Config( CLK_PSADIV_t PSAfactor,
                               CLK_PSBCDIV_t PSBCfactor );
uint8_t CLKSYS_Main_ClockSource_Select( CLK_SCLKSEL_t clockSource );
void CLKSYS_RTC_ClockSource_Enable( CLK_RTCSRC_t clockSource );
void CLKSYS_AutoCalibration_Enable( uint8_t clkSource, bool extReference );
void CLKSYS_XOSC_FailureDetection_Enable( void );
void CLKSYS_Configuration_Lock( void );


/*
--------------------------------------------------------------------------
OSC - Oscillator
--------------------------------------------------------------------------

from XMEGA192A3 Header file



  Oscillator  
typedef struct OSC_struct
{
    register8_t CTRL;    Control Register  
    register8_t STATUS;    Status Register  
    register8_t XOSCCTRL;    External Oscillator Control Register  
    register8_t XOSCFAIL;    External Oscillator Failure Detection Register  
    register8_t RC32KCAL;    32kHz Internal Oscillator Calibration Register  
    register8_t PLLCTRL;    PLL Control REgister  
    register8_t DFLLCTRL;    DFLL Control Register  
} OSC_t;

  Oscillator Frequency Range  
typedef enum OSC_FRQRANGE_enum
{
    OSC_FRQRANGE_04TO2_gc = (0x00<<6),    0.4 - 2 MHz  
    OSC_FRQRANGE_2TO9_gc = (0x01<<6),    2 - 9 MHz  
    OSC_FRQRANGE_9TO12_gc = (0x02<<6),    9 - 12 MHz  
    OSC_FRQRANGE_12TO16_gc = (0x03<<6),    12 - 16 MHz  
} OSC_FRQRANGE_t;

  External Oscillator Selection and Startup Time  
typedef enum OSC_XOSCSEL_enum
{
    OSC_XOSCSEL_EXTCLK_gc = (0x00<<0),    External Clock - 6 CLK  
    OSC_XOSCSEL_32KHz_gc = (0x02<<0),    32kHz TOSC - 32K CLK  
    OSC_XOSCSEL_XTAL_256CLK_gc = (0x03<<0),    0.4-16MHz XTAL - 256 CLK  
    OSC_XOSCSEL_XTAL_1KCLK_gc = (0x07<<0),    0.4-16MHz XTAL - 1K CLK  
    OSC_XOSCSEL_XTAL_16KCLK_gc = (0x0B<<0),    0.4-16MHz XTAL - 16K CLK  
} OSC_XOSCSEL_t;

  PLL Clock Source  
 
typedef enum OSC_PLLSRC_enum
{
    OSC_PLLSRC_RC2M_gc = (0x00<<6),    Internal 2MHz RC Oscillator  
    OSC_PLLSRC_RC32M_gc = (0x02<<6),    Internal 32MHz RC Oscillator  
    OSC_PLLSRC_XOSC_gc = (0x03<<6),    External Oscillator  
} OSC_PLLSRC_t;
 

  System Clock Selection  
typedef enum CLK_SCLKSEL_enum
{
    CLK_SCLKSEL_RC2M_gc = (0x00<<0),    Internal 2MHz RC Oscillator  
    CLK_SCLKSEL_RC32M_gc = (0x01<<0),    Internal 32MHz RC Oscillator  
    CLK_SCLKSEL_RC32K_gc = (0x02<<0),    Internal 32kHz RC Oscillator  
    CLK_SCLKSEL_XOSC_gc = (0x03<<0),    External Crystal Oscillator or Clock  
    CLK_SCLKSEL_PLL_gc = (0x04<<0),    Phase Locked Loop  
} CLK_SCLKSEL_t;

  Prescaler A Division Factor  
typedef enum CLK_PSADIV_enum
{
    CLK_PSADIV_1_gc = (0x00<<2),    Divide by 1  
    CLK_PSADIV_2_gc = (0x01<<2),    Divide by 2  
    CLK_PSADIV_4_gc = (0x03<<2),    Divide by 4  
    CLK_PSADIV_8_gc = (0x05<<2),    Divide by 8  
    CLK_PSADIV_16_gc = (0x07<<2),    Divide by 16  
    CLK_PSADIV_32_gc = (0x09<<2),    Divide by 32  
    CLK_PSADIV_64_gc = (0x0B<<2),    Divide by 64  
    CLK_PSADIV_128_gc = (0x0D<<2),    Divide by 128  
    CLK_PSADIV_256_gc = (0x0F<<2),    Divide by 256  
    CLK_PSADIV_512_gc = (0x11<<2),    Divide by 512  
} CLK_PSADIV_t;

  Prescaler B and C Division Factor  
typedef enum CLK_PSBCDIV_enum
{
    CLK_PSBCDIV_1_1_gc = (0x00<<0),    Divide B by 1 and C by 1  
    CLK_PSBCDIV_1_2_gc = (0x01<<0),    Divide B by 1 and C by 2  
    CLK_PSBCDIV_4_1_gc = (0x02<<0),    Divide B by 4 and C by 1  
    CLK_PSBCDIV_2_2_gc = (0x03<<0),    Divide B by 2 and C by 2  
} CLK_PSBCDIV_t;

  RTC Clock Source  
typedef enum CLK_RTCSRC_enum
{
    CLK_RTCSRC_ULP_gc = (0x00<<1),    1kHz from internal 32kHz ULP  
    CLK_RTCSRC_TOSC_gc = (0x01<<1),    1kHz from 32kHz crystal oscillator on TOSC  
    CLK_RTCSRC_RCOSC_gc = (0x02<<1),    1kHz from internal 32kHz RC oscillator  
    CLK_RTCSRC_TOSC32_gc = (0x05<<1),    32kHz from 32kHz crystal oscillator on TOSC  
} CLK_RTCSRC_t;


*/
#endif
